// DacsClient.h: interface for the DacsClient class.
//
// Disclaimer: This sample code has been provided for teaching
// and training purposes only. It is not intended for use
// within production applications.

#ifndef DACSCLIENTS_H
#define DACSCLIENTS_H

#include "Common/RFA_Vector.h"

#include "Common/Client.h"
#include "Common/EventQueue.h"
#include "Common/StandardPrincipalIdentity.h"
#include "Common/Handle.h"
#include "Common/Buffer.h"
#include "Dacs/AuthSystem.h"
#include "Dacs/AuthAgent.h"
#include "Dacs/AuthLock.h"

#include "rtr/rsslTypes.h"


class DacsAppClient;
class MarketDataClient;
class StarterConsumer;


// **************** DacsClient ***************************
/**
	DacsClient class, which is an implement of Client interface,
	receives Open DACS related notifications.
 */

class DacsClient :
	public rfa::common::Client
{
public:
	DacsClient(rfa::common::EventQueue &eventQ, rfa::common::StandardPrincipalIdentity& spId,rfa::common::RFA_String& dacsConnectionStr);
	virtual ~DacsClient();
	int init();
	rfa::dacs::AuthorizationAgent::AuthorizationConnection ConnectionStatus();
	void processEvent(const rfa::common::Event& event);
	
	bool checkSubSBE(const rfa::common::RFA_String& serviceName, const rfa::common::RFA_String& itemName);
	bool checkSubCBE( const rfa::common::RFA_String& serviceName, 
		const rfa::common::RFA_String& itemName, const rfa::common::RFA_Vector<unsigned long>& peList, rfa::dacs::AuthorizationLock::Operator op, bool reperm = false);
	bool checkSubCBE( const rfa::common::RFA_String& serviceName, 
		const rfa::common::RFA_String& itemName, const rfa::common::Buffer* permData, bool reperm = false);
	bool checkSubCBE( int serviceId, 
		const rfa::common::RFA_String& itemName, const rfa::common::Buffer* permData, bool reperm = false);

	// Keeping track of login request status
	enum LoginEnum
	{
		LoginPending = 0,
		LoginFailed = 1,
		LoginSucceeded = 2
	};

	LoginEnum getLoginStatus() {return _loginStatus;};

protected:
	// return codes for DACS Authorization calls
	static const int DACSAUTH_SUCCESS = 0;
	static const int DACSAUTH_ERROR = -1;
		
private:
	void cleanup();
	void processAuthorizationEvent
				(const rfa::dacs::AuthorizationAgentEvent& authAgentEvent);
	void printStatus(rfa::dacs::AuthorizationAgentEventStatus& status);
	void decodeDacsLock( const unsigned char* lock, int len);

	rfa::dacs::AuthorizationSystem* _authSys;
	rfa::dacs::AuthorizationAgent* _authAgent;
	rfa::common::RFA_String& _dacsConnectionStr;
	rfa::common::EventQueue& _pEventQ;
	//DacsAppClient& _appClient;
	rfa::common::StandardPrincipalIdentity& _spId;
	rfa::dacs::AuthorizationSystem::AUTH_SYSTEM_OPTION_VEC options;
	rfa::dacs::AuthorizationSystem::AUTH_SYSTEM_OPTION_ELEMENT optionsElement;

	rfa::common::Handle* _phLogin;
	LoginEnum _loginStatus;
};

//void decodeDacsLock( const unsigned char* lock, int len);
//void decodeDacsLock( const rfa::common::Buffer& buf );

#endif // DACSCLIENTS_H
